/* The Type of Track representing an audiotrack located on a cdrom */

#include <sys/wait.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include "tracks.h"
#include "piping.h"
#include "preferences.h"
#include "varmanwidgets.h"
#include "varman.h"
#include "audiotrack.h"
#include "cdromlow.h"

int audiotrack_openpipe(void*trackinfo)
{	
	int inp,outp,errp;
	char *call;
	audiotrack_info *i;
	
	i=(audiotrack_info*)((tracks_trackinfo*)trackinfo)->data;
	call=varman_getvar_copy(global_defs,"audiotrack_read");
	varman_replacevars(global_defs,call);
	varman_replacestring(call,"$device",(char*)&i->device);
	varman_replacestringbyfloat(call,"$tracknum",i->tracknum);
	i->cdda2wavpid=piping_create(call,&inp,&outp,&errp);
	i->cdda2wavpipe=outp;
	free(call);
	return outp;
}
;

void audiotrack_closepipe(void*trackinfo)
{
	audiotrack_info *i;
	
	i=(audiotrack_info*)((tracks_trackinfo*)trackinfo)->data;
	close(i->cdda2wavpipe);
	waitpid(i->cdda2wavpid,&i->cdda2wavstatus,0);
	/* wait for client to finish */
}
;

int audiotrack_tracksize(void*trackinfo)
{
	audiotrack_info *i;
	
	i=(audiotrack_info*)((tracks_trackinfo*)trackinfo)->data;

	return i->tracksize;
}
;

int audiotrack_dataavail(void*trackinfo)
{
	audiotrack_info *i;
	int dataavail;

	i=(audiotrack_info*)((tracks_trackinfo*)trackinfo)->data;
	dataavail=1;
	if (waitpid(i->cdda2wavpid,&i->cdda2wavstatus,WNOHANG)!=0)
	    dataavail=0;

	return dataavail;
}
;	

/* FIXME: do implement some "track present" check here */
int  audiotrack_valid(void*trackinfo)
{
	return 1;
}
;

void audiotrack_destroy(void*trackinfo)
{
	audiotrack_info *i;
	
	i=(audiotrack_info*)((tracks_trackinfo*)trackinfo)->data;
	
	if (audiotrack_dataavail(trackinfo))
	    close (i->cdda2wavpipe);
	free(i);
}
;

tracks_trackinfo *audiotrack_create(char *device,int tracknum)
{
	tracks_trackinfo *at;
	audiotrack_info *info;
	char name[255];
	
	info=(audiotrack_info*)malloc(sizeof(audiotrack_info));
	strcpy((char*)&info->device,device);
	info->tracknum=tracknum;
	info->tracksize=cdromlow_tracksize(device,tracknum);
	
	strcpy((char*)&name,"CD ");
	sprintf(&name[strlen((char*)&name)],"%#x",cdromlow_cddbnumber(device));
	strcat((char*)&name," Audiotrack ");
	sprintf(&name[strlen((char*)&name)],"%d",info->tracknum);
	
	at=tracks_create((char*)&name,audio,atadd,
			 audiotrack_openpipe,
			 audiotrack_closepipe,
			 audiotrack_tracksize,
			 audiotrack_dataavail,
			 audiotrack_valid,
			 audiotrack_destroy,
			 info);
	return at;
}
;


/* configure the cdda audio stream from cdrom tracktype */
void audiotrack_init()
{	
	GtkWidget *audiotrack_prefs;
	GtkWidget *label;
	
	varmanwidgets_widget *atc;
	varmanwidgets_widget *read;
	
	audiotrack_prefs=gtk_vbox_new(0,0);
	gtk_widget_show(audiotrack_prefs);
	label=gtk_label_new("CD Digital Audio");
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(preferences_editpad),
				 audiotrack_prefs,
				 label);
	
	atc=varmanwidgets_entry_new("Audio track Client:",
				    "audiotrack_client",
				    global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(audiotrack_prefs),atc->visual,0,0,0);

	read=varmanwidgets_entry_new("Audio track reading:",
				     "audiotrack_read",
				     global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(audiotrack_prefs),read->visual,0,0,0);
	
}
;
	
	
				    	
