/* Some functions to handle lists of files as given back on drag and drop
 * events or popup events (delete,rename,etc) */

#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <dirent.h>

#include <gtk/gtk.h>

#include "dircache.h"
#include "fileman.h"

/* the following function can be used to parse a selection as given back
 * by drag and drop and call filehandle(char*) for each separate file */
int fileman_selectionhandler(char *s,char *p,int (*filehandle)(char *s,char *p))
{	
	char *item,*newitem,*chr10,*chr13;
	char filename[256];
	int  success;      // does the file system need reloading ??

	success=0;
	item=s;
	do
	  {
		  chr13=strchr(item,13);
		  chr10=strchr(item,10);
		  newitem=chr13;
		  if (((chr10!=NULL) && (chr10<chr13)) || (chr13==NULL))
		      newitem=chr13;
		  if (newitem==NULL) 
		      newitem=item+strlen(item);
		  else
		    {			    
			    *newitem=0;
			    do 
			      {
				      newitem++;
			      }
			    while ((*newitem==10)||(*newitem==13));
		    }
		  ;
		  strcpy((char*)&filename,item);
		  if (strlen((char*)&filename)!=0)
		      success|=filehandle((char*)&filename,p);
		  item=newitem;
	  }
	while (strlen((char*)&filename)>0);
	return success;
}
;

/* get filename without path. if resulting filename is "" as it might happen
 * when the root dir gets added to a cd,it is instead called "root"
 * char *d is expected to be a pointer to an already allocated memory region */
void fileman_strippath(char *filename,char *stripped)
{
	char *d,*dirname;
	
	d=(char*)malloc(1024);
	strcpy(d,filename);
	if (d[strlen(d)-1]=="/"[0]) /* do not allow slash at the end */
	    d[strlen(d)-1]=0;       /* this could otherwise mess up the function */
	
	dirname=d;
	while (strchr(dirname,"/"[0])!=NULL)
	    dirname=strchr(dirname,"/"[0])+1;
		
	strcpy(stripped,dirname);
	if (strlen(stripped)==0) 
	    strcpy(stripped,"root");
	free(d);
}
;	

/* remove the specified single file item. If this is a directory,it will
 * be removed recursively. Parameter char *p is unused and set to NULL */
int fileman_remove_item(char *s,char *p)
{
	char *file;
	int  success;

	file=strchr(s,":"[0])+1;
	success=0;
	if (dircache_isdir(file))
	  {
		  success=dirlow_removedir(file);
		  if (!success) perror("couldnt remove directory");
	  }	
	else
	  {		  
		  success=(remove(file)==0);
		  if (!success) perror("Couldnt remove file");
	  }
	;
	return success;
}
;	

/* remove a whole batch of files using the function above */
int fileman_remove_items(char *s)
{
	return fileman_selectionhandler(s,NULL,fileman_remove_item);
}
;

/* add a single normal file to the path specified by char *p.
 * this is done by creating a symbolic link to the source file in
 * the destination directory */
int fileman_addnormalfile(char *f,char *p)
{
	char *filename;
	char *destination;
	int success;
	
	success=1;
	filename=(char*)malloc(1024);
	fileman_strippath(f,filename);
	destination=(char*)malloc(1024);
	strcpy(destination,p);
	if (destination[strlen(destination)-1]!="/"[0])
	    strcat (destination,"/");
	strcat(destination,filename);
	if (symlink(f,destination)==-1)
	  {		  
		  success=0;
		  perror("Couldnt create Symlink");
	  }
	;
	free(filename);
	/* FIXME: print some nice message box here ;-) */
	return success;
}
;

/* add a single directory to the path specified by char *p
 * all the items of this directory will be added depending on wether theyre
 * directories themselves,in which case adddirectory will simply be called
 * recursively,or wether theyre just normal files,which will then be added
 * using addnormalfile() */
int fileman_adddirectory(char *sd,char *p)
{
	char *dirname;
	char *destdir;
	
	char *sourcefile;
	DIR  *dir;
	struct dirent *entry;

	dirname=(char*)malloc(1024);
	fileman_strippath(sd,dirname);
	destdir=(char*)malloc(1024);
	strcpy(destdir,p);
	if (destdir[strlen(destdir)-1]!="/"[0])
	    strcat (destdir,"/");
	strcat(destdir,dirname);
	if (mkdir(destdir,0777)==-1)
	  {
		  perror("Couldnt create directory");
	  }
	;
	sourcefile=(char*)malloc(1024);
	
	dir=opendir(sd);
	if (dir!=NULL)
	  {		  	
		  while ((entry=readdir(dir))!=NULL)
		    {
			    if ( strcmp((char*)&entry->d_name,"..") &&
				strcmp((char*)&entry->d_name,"."))
			      {				      
				      strcpy(sourcefile,sd);
				      if (sd[strlen(sd)-1]!="/"[0])
					  strcat(sourcefile,"/");
				      strcat(sourcefile,(char*)&entry->d_name);
				      if (dircache_isdir(sourcefile))
					  fileman_adddirectory(sourcefile,destdir);
				      else
					  fileman_addnormalfile(sourcefile,destdir);
			      }
			    ;
		    }
		  ;
		  closedir (dir);
	  }
	;
	
	free(dirname);
	return 1;
}
;

/* add a single generic file item to the patch specified by char *p 
 * this can be either a normal file or a directory */
int fileman_addfile(char*f,char*p)
{
	int changed;
	
	changed=0;
	if (dircache_isdir(f))
	  {
		  changed=fileman_adddirectory(f,p);
	  }
	else
	  {
		  changed=fileman_addnormalfile(f,p);
	  }
	;
	return changed;
}
;

/* will be used to copy the contents of a trackstream into a file
 * within the specified path with the name of the stream as filename
 * This function is currently not implemented and will generate a 
 * dummy message only describing the stream it has been called with */
int fileman_addstream(char*s,char *p)
{
	printf ("copy stream-content of %s to %s\n",s,p);
	return 0;
};


/* add an item to the specified path. item can be one of stream,directory,
 * normal file */
int fileman_add_item(char *s,char *p)
{
	int success;
	success=0;
	if (!strncmp("file:",
		     s,
		     5))
	    success|=fileman_addfile(strchr(s,":"[0])+1,p);
	if (!strncmp("stream:",
		     s,
		     7))
	    success|=fileman_addstream(strchr(s,":"[0])+1,p);
	return success;
}
;


int fileman_add_items(char *s,char *p)
{	
	return fileman_selectionhandler(s,p,fileman_add_item);
}
;

typedef struct 
{
	GtkWidget *dlwin;
	GtkWidget *dirname;
	void(*finished)(gpointer);
	gpointer data;
	char path[1024];
} fileman_mkdir_info;

void fileman_mkdir_gotname(GtkWidget *w,gpointer data)
{
	fileman_mkdir_info *info;
	char *name;
	
	info=(fileman_mkdir_info*)data;
	name=gtk_entry_get_text(GTK_ENTRY(info->dirname));
	if (strlen(name)>0)
	  {		  
		  dirlow_mkdirin((char*)&info->path,name);
		  info->finished(info->data);
	  }
	;
	gtk_grab_remove(info->dlwin);
	gtk_widget_destroy(info->dlwin);
}
;

void fileman_mkdir_cancel(GtkWidget *w,gpointer data)
{
	fileman_mkdir_info *info;
	
	info=(fileman_mkdir_info*)data;	
	gtk_grab_remove(info->dlwin);
	gtk_widget_destroy(info->dlwin);
}
;
	

/* display a dialog asking for a directory name,create requested directory
 * then */
void fileman_mkdir_dialog(char *path,
			  void(*finished)(gpointer),
			  gpointer data)
{
	fileman_mkdir_info *info;
	GtkWidget *ok;
	GtkWidget *cancel; /* clicking cancel will not call "finished" */
	
	info=(fileman_mkdir_info*)malloc(sizeof(fileman_mkdir_info));
	strcpy((char*)&info->path,path);
	info->finished=finished;
	info->data=data;

	info->dlwin=gtk_dialog_new();
	gtk_grab_add(info->dlwin);
	
	info->dirname=gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(info->dlwin)->vbox),
			   info->dirname,TRUE,TRUE,5);
	gtk_widget_show(info->dirname);
	
	ok=gtk_button_new_with_label("Ok");
	gtk_signal_connect(GTK_OBJECT(ok),"clicked",
			   fileman_mkdir_gotname,info);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(info->dlwin)->action_area),
			   ok,FALSE,FALSE,0);
	gtk_widget_show(ok);
	
	cancel=gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(cancel),"clicked",
			   fileman_mkdir_cancel,info);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(info->dlwin)->action_area),
			   cancel,FALSE,FALSE,0);
	gtk_widget_show(cancel);
	
	gtk_window_set_title(GTK_WINDOW(info->dlwin),"Create directory");
	gtk_window_set_position(GTK_WINDOW(info->dlwin),GTK_WIN_POS_CENTER);
	gtk_widget_set_usize(info->dlwin,200,80);
	gtk_widget_show(info->dlwin);
}
;
	
	
	
				   
	
	
