/* The Type of Track representing the Fs edit stuff */

#include <sys/wait.h>
#include <unistd.h>
#include <stdio.h>

#include "tracks.h"
#include "piping.h"
#include "preferences.h"
#include "varmanwidgets.h"
#include "isotrack.h"
#include "colors.h"

int isotrack_openpipe(void*trackinfo)
{	
	int inp,outp,errp;
	char *call;
	isotrack_info *i;
	
	i=(isotrack_info*)((tracks_trackinfo*)trackinfo)->data;	
	call=varman_getvar_copy(global_defs,"isotrack_client");
	varman_replacevars(global_defs,call);
	i->mkisofspid=piping_create(call,&inp,&outp,NULL);
	i->mkisofspipe=outp;
	free(call);
	return outp;
}
;

void isotrack_closepipe(void*trackinfo)
{
	isotrack_info *i;
	
	i=(isotrack_info*)((tracks_trackinfo*)trackinfo)->data;
	close(i->mkisofspipe);
	waitpid(i->mkisofspid,&i->mkisofsstatus,0); // wait for client to finish
}
;

int isotrack_tracksize(void*trackinfo)
{
	int inp,outp;
	char *call;
	isotrack_info *i;
	FILE *stream;
	int  tracksize;
	int  mkisofspid,mkisofsstatus; /* create a new copy here to be "thread-safe" */
	
	i=(isotrack_info*)((tracks_trackinfo*)trackinfo)->data;
	call=varman_getvar_copy(global_defs,"isotrack_tsclient");
	varman_replacevars(global_defs,call);
	mkisofspid=piping_create(call,&inp,&outp,NULL);	
	stream=fdopen(outp,"r");
	fscanf(stream,"%d",&tracksize);
	fclose(stream);
	free(call);
	waitpid(mkisofspid,&mkisofsstatus,0); // wait for client to finish
	
	return tracksize;
}
;

int isotrack_dataavail(void*trackinfo)
{
	isotrack_info *i;
	int dataavail;

	i=(isotrack_info*)((tracks_trackinfo*)trackinfo)->data;
	dataavail=1;
	if (waitpid(i->mkisofspid,&i->mkisofsstatus,WNOHANG)!=0)
	    dataavail=0;

	return dataavail;
}
;	

int  isotrack_valid(void*trackinfo)  /* isotrack is always valid */
{
	return 1;
}
;

void isotrack_destroy(void*trackinfo)
{
	isotrack_info *i;
	
	i=(isotrack_info*)((tracks_trackinfo*)trackinfo)->data;
	
	if (isotrack_dataavail(trackinfo))
	    close (i->mkisofspipe);
	free(i);
}
;

tracks_trackinfo *isotrack_create()
{
	tracks_trackinfo *it;
	isotrack_info *info;
	
	info=(isotrack_info*)malloc(sizeof(isotrack_info));	
	
	it=tracks_create("Filesystem (ISO)",data,none,
			 isotrack_openpipe,
			 isotrack_closepipe,
			 isotrack_tracksize,
			 isotrack_dataavail,
			 isotrack_valid,
			 isotrack_destroy,
			 info);
	return it;
}
;

/* this is called by main.c to do some init stuff,e.g. creating the preferences
 * entry */
void isotrack_init()
{
	GtkWidget *isotrack_prefs;
	GtkWidget *label;
	
	varmanwidgets_widget *path;
	varmanwidgets_widget *itc;
	varmanwidgets_widget *tracksizedet;
	
	isotrack_prefs=gtk_vbox_new(0,0);
	gtk_widget_show(isotrack_prefs);
	label=gtk_label_new("ISO9660");
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(preferences_editpad),
				 isotrack_prefs,
				 label);
	
	path=varmanwidgets_entry_new("CDfs Area",
				     "fsedit_destpath",
				     global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(isotrack_prefs),path->visual,0,0,0);
	
	itc=varmanwidgets_entry_new("Create ISO Track with",
				    "isotrack_client",
				    global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(isotrack_prefs),itc->visual,0,0,0);

	tracksizedet=varmanwidgets_entry_new("Determine tracksize with",
				    "isotrack_tsclient",
				    global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(isotrack_prefs),tracksizedet->visual,0,0,0);
}
;
	
