/* piping functions for data filters and cd writing */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <fcntl.h>

#include "piping.h"

#define FDREAD 0
#define FDWRITE 1

int piping_pidwatch_callback(gpointer data)
{
	piping_watch *w;
	int callagain;

	w=(piping_watch*)data;
	if (waitpid(w->pid,&w->status,WNOHANG)!=0) 
	  {
		  callagain=w->finishcb(w->status,w->data);
	  }
	else 
	    callagain=1;
	if (callagain==0) piping_pidwatch_remove(w);
	
	return callagain;
}
;

piping_watch *piping_pidwatch_add(int pid,
				  piping_finishcb callback,
				  gpointer data)
{
	piping_watch *w;
	
	w=(piping_watch*)malloc(sizeof(piping_watch));
	w->pid=pid;
	w->status=0;
	w->finishcb=callback;
	w->data=data;
	w->handle=gtk_timeout_add(PIPING_PIDWATCH_TIMEOUT,
				  piping_pidwatch_callback,
				  (gpointer)w);
	
	return w;
}
;

void piping_pidwatch_remove(piping_watch *w)
{
	gtk_timeout_remove(w->handle);
	free(w);
}
;

int piping_create(char *command,int *in,int *out,int *err) // command: program to start and parameters separated by spaces 
{
	int inpipe[2];
	int outpipe[2];
	int errpipe[2];

	char *args[PIPING_MAX_ARGS];
	char *argstring;
	char *currentpos;
	int  x;
	
	int childpid;	
		
	argstring=(char*)malloc(strlen(command)+1);
	strcpy(argstring,command);  // make a copy first,then modify it appropriately
	
	x=0;currentpos=argstring;
	do 
	  {
		  args[x]=currentpos;
		  currentpos=strchr(currentpos,32); // search for space
		  if (currentpos!=0)
		    {			   
			    *currentpos=0;
			    currentpos++;
		    }
		  ;
		  x++;
	  }
	while (currentpos!=0);
	args[x]=0;
	
	if (in!=NULL)
	    pipe(inpipe);
	else
	  {		  
		  inpipe[FDREAD]=open("/dev/zero",O_RDONLY);
		  inpipe[FDWRITE]=open("/dev/zero",O_WRONLY);
	  }
	;
	if (out!=NULL)
	    pipe(outpipe);
	else
	  {		  
		  outpipe[FDREAD]=open("/dev/zero",O_RDONLY);
		  outpipe[FDWRITE]=open("/dev/zero",O_WRONLY);
	  }
	;
	if (err!=NULL)	    
	    pipe(errpipe);
	else
	  {		  
		  errpipe[FDREAD]=open("/dev/zero",O_RDONLY);
		  errpipe[FDWRITE]=open("/dev/zero",O_WRONLY);
	  }
	;
	childpid=fork();
	switch (childpid)
	  {
	  case 0:
		  close (inpipe[FDWRITE]);  // handle IO for client
		  close (0);
		  dup(inpipe[FDREAD]);
		  close(inpipe[FDREAD]);
		  
		  close (outpipe[FDREAD]);
		  close (1);
		  dup(outpipe[FDWRITE]);
		  close(outpipe[FDWRITE]);
		  
		  close(errpipe[FDREAD]);
		  close (2);
		  dup(errpipe[FDWRITE]);
		  close(errpipe[FDWRITE]);
		  
		  execvp(args[0],args);
		  perror ("couldnt run client");
		  free(argstring);
		  exit(-2);
	  case -1:
		  perror ("forking error");
		  free(argstring);
		  exit(-1);
	  default:
		  close(inpipe[FDREAD]);
		  close(outpipe[FDWRITE]);
		  close(errpipe[FDWRITE]);
		  if (in!=NULL)  
		      *in=inpipe[FDWRITE];
		  else
		      close(inpipe[FDWRITE]);		  
		  if (out!=NULL) 
		      *out=outpipe[FDREAD];
		  else
		      close(outpipe[FDREAD]);
		  if (err!=NULL) 
		      *err=errpipe[FDREAD];
		  else
		      close(errpipe[FDREAD]);
		  free(argstring);
		  return childpid;
	  }
	;
}
;
		  
		  
		  
			
	

		  
		  
	
	
