/* Recording controls */

#include <gtk/gtk.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/types.h>

#include "preferences.h"
#include "main.h"
#include "colors.h"
#include "record.h"
#include "piping.h"
#include "tracks.h"
#include "trackedit.h"
#include "rterm.h"
#include "datacopydlg.h"
#include "varman.h"
#include "varmanwidgets.h"

int record_active;  // becomes true whenever access to the cdwriter is being performed

/* The following vars are not being exported to other objects,
 * so they do not have the leading "record_" ... */

int inpipe;
int outpipe;
int errpipe;
	
rterm_pipeinfo *outtag;
rterm_pipeinfo *errtag;

int trackpipe;

int cdrecordpid;
int cdrecordstatus;
int cdrecordinpipetag;

char readbuf[RECORD_BUFSIZE];
int  readcount;

int  currenttrack;
datacopydlg_dlginfo *dlg;

/* End of local var list */

void record_unexpected_childexit()
{
	rterm_printcln(Blue,"Child exited unexpectedly.");
}
;

int record_fixating_finished(int status,gpointer data)
{	
	rterm_disconnectpipe(outtag);
	rterm_disconnectpipe(errtag);
	close(inpipe);
	close(outpipe);
	close(errpipe);
	
	rterm_printcln(Blue,"CD recording process finished.");
	
	record_active=0;	    
	return 0;
}
;

void record_finishrecord()
{
	piping_watch *fixrom;
	char *call;
	
	rterm_printcln (Blue,"fixating CDROM.");
	call=varman_getvar_copy(global_defs,"rec_fixate");
	varman_replacevars(global_defs,call);
	cdrecordpid=piping_create(call,&inpipe,&outpipe,&errpipe);
	free(call);
	outtag=rterm_connectpipe(&Black,outpipe);
	errtag=rterm_connectpipe(&Red,errpipe);
	fixrom=piping_pidwatch_add(cdrecordpid,
				   record_fixating_finished,
				   NULL);
}
;

void record_nexttrack();

void record_writetrack(gpointer data,
		       gint source,
		       GdkInputCondition condition)
{
	readcount=read(trackpipe,(char*)&readbuf,RECORD_BUFSIZE);
	if (waitpid(cdrecordpid,&cdrecordstatus,WNOHANG)!=0) 
	  {
		  record_unexpected_childexit();
		  readcount=0; // exit writing process
	  }
	;
	if (readcount>0)
	  {
		  write(inpipe,(char*)&readbuf,readcount);
		  datacopydlg_newdatablock(dlg,readcount);
	  }
	else  // current track is done
	  {
		  gdk_input_remove(cdrecordinpipetag);
		  
		  tracks_closepipe(trackedit_gettrack(currenttrack));
		  close(inpipe);
		  waitpid(cdrecordpid,&cdrecordstatus,0); // wait for cdrecord to exit.
		  rterm_disconnectpipe(outtag);
		  rterm_disconnectpipe(errtag);
		  close(outpipe);
		  close(errpipe);

		  datacopydlg_destroy(dlg);
		  
		  currenttrack++;
		  if (currenttrack<trackedit_tracks)
		    {
			    record_nexttrack();
		    }
		  else 
		    {
			    record_finishrecord();
		    }
		  ;
	  }
	;
}
;
	
void record_nexttrack()
{     
	char *call;
	
	/* do record only tracks that are available */
	if (tracks_valid(trackedit_gettrack(currenttrack)))
	  {		  	
		  trackpipe=tracks_openpipe(trackedit_gettrack(currenttrack));
	
		  call=varman_getvar_copy(global_defs,"rec_writetrack");
		  varman_replacevars(global_defs,call);		  
		  switch (trackedit_gettrack(currenttrack)->tracktype)
		    {			    
		    case data:
			    varman_replacestring(call,"$tracktype","data");
			    break;
		    case audio:
			    varman_replacestring(call,"$tracktype","audio");
			    break;
		    }
		  ;
		  cdrecordpid=piping_create(call,&inpipe,&outpipe,&errpipe);
		  outtag=rterm_connectpipe(&Black,outpipe);
		  errtag=rterm_connectpipe(&Red,errpipe);

		  strcpy(call,"writing track #");
		  sprintf(&call[strlen(call)],
			  "%d",
			  currenttrack);
		  dlg=datacopydlg_create(call,
					 tracks_tracksize(trackedit_gettrack(currenttrack)),
					 NULL,NULL);
		  free(call);
		  
		  cdrecordinpipetag=gdk_input_add (inpipe,
						   GDK_INPUT_WRITE,
						   record_writetrack,
						   NULL);
	  }
	else
	  {
		  currenttrack++;
		  if (currenttrack<trackedit_tracks)
		    {
			    record_nexttrack();
		    }
		  else 
		    {
			    record_finishrecord();
		    }
		  ;
	  }
	;
}
;

void record_startrecord(GtkWidget *w)
{
	int x,discsize;
	char dscount[256];
	char buf[256];
	
	if (record_active) return;
	record_active=1;

	discsize=0;
	for (x=0;x<trackedit_tracks;x++)
	  {
		  discsize+=tracks_tracksize(trackedit_gettrack(x));

/* FIXME: create some well implemented preread function
 * using the callback structure introduced in precachetrack.c 
		  if (trackedit_gettrack(x)->precacherequired>=atwrite)
		      trackedit_precache(x);*/
	  }
	;
		  
	strcpy((char*)&buf,"Recording ");
	sprintf((char*)&dscount,"%i",discsize);
	strcat((char*)&buf,(char*)&dscount);
	strcat((char*)&buf," bytes to CD");
	rterm_printcln(Blue,(char*)&buf);

	currenttrack=0;

	if (trackedit_tracks>0)
	    record_nexttrack();
	else
	  {		  
		  rterm_printcln(Blue,"No recordable Tracks found.");
		  record_active=0;
	  }
	;
}
;
	

int record_clear_finished(int status,gpointer data)
{
	rterm_disconnectpipe(outtag);
	rterm_disconnectpipe(errtag);
	
	close(inpipe);
	close(outpipe);
	close(errpipe);

	rterm_printcln(Blue,"CDRW blanking complete.");
	
	record_active=0;
	return 0;
}
;

void record_clear(GtkWidget *w)
{
	piping_watch *cdrecordwatch;
	char *call;

	if (record_active) return; // dont run 2 cdwrite processes simultaneously	
	record_active=1;
	
	call=varman_getvar_copy(global_defs,"rec_cleardisc");
	varman_replacevars(global_defs,call);	
	rterm_printcln (Blue,"blanking CDRW.");
	cdrecordpid=piping_create(call,&inpipe,&outpipe,&errpipe);
	free(call);
	outtag=rterm_connectpipe(&Black,outpipe);
	errtag=rterm_connectpipe(&Red,errpipe);

	cdrecordwatch=piping_pidwatch_add(cdrecordpid,
					  record_clear_finished,
					  NULL);
}
;
		
GtkWidget *record_create()
{
	GtkWidget *rec;
	
	GtkWidget *hbox_record;
	GtkWidget *frame_record;
	GtkWidget *button_record;
	GtkWidget *button_multisession;

	GtkWidget *hbox_cdrw;
	GtkWidget *frame_cdrw;
	GtkWidget *button_clear;
	varmanwidgets_widget *button_cleartype;
	
	GtkWidget *terminal;

	GtkWidget *label;
	GtkWidget *record_prefs;
	
	varmanwidgets_widget *edit;
	
	rec=gtk_table_new(2,2,TRUE);
	
	frame_record=gtk_frame_new("Recording");
	hbox_record=gtk_hbox_new(0,0);
	
	button_record=gtk_button_new_with_label("Record");
	gtk_signal_connect (GTK_OBJECT(button_record),"clicked",
			    GTK_SIGNAL_FUNC (record_startrecord),NULL);
	gtk_box_pack_start(GTK_BOX(hbox_record),button_record,0,0,0);
	gtk_widget_show(button_record);

	button_multisession=gtk_check_button_new_with_label("Multisession");
	gtk_box_pack_start(GTK_BOX(hbox_record),button_multisession,0,0,0);
	gtk_widget_show(button_multisession);
	
	gtk_container_add(GTK_CONTAINER(frame_record),hbox_record);
	gtk_widget_show(hbox_record);
	gtk_table_attach(GTK_TABLE(rec),frame_record,
			 0,1,0,1,
			 0,0,0,0);
	gtk_widget_show(frame_record);

	frame_cdrw=gtk_frame_new("CD/RW");
	hbox_cdrw=gtk_hbox_new(0,0);
	
	button_clear=gtk_button_new_with_label("Clear Disc");
	gtk_signal_connect (GTK_OBJECT(button_clear),"clicked",
			    GTK_SIGNAL_FUNC (record_clear),NULL);
	gtk_box_pack_start(GTK_BOX(hbox_cdrw),button_clear,0,0,0);
	gtk_widget_show(button_clear);

	button_cleartype=varmanwidgets_checkbox_new("TOC only",
						    "rec_blankingmode",
						    global_defs,
						    APPLYMODE_ALWAYS,
						    0,
						    "fast",
						    "all"
						    );
	gtk_box_pack_start(GTK_BOX(hbox_cdrw),button_cleartype->visual,0,0,0);
	gtk_widget_show(button_cleartype->visual);
	
	gtk_container_add(GTK_CONTAINER(frame_cdrw),hbox_cdrw);
	gtk_widget_show(hbox_cdrw);
	gtk_table_attach(GTK_TABLE(rec),frame_cdrw,
			 0,1,1,2,
			 0,0,0,0);
	gtk_widget_show(frame_cdrw);

	
	terminal=rterm_create();
       	gtk_table_attach_defaults(GTK_TABLE(rec),terminal,
				  1,2,0,2);
	gtk_widget_show(terminal);

	record_prefs=gtk_vbox_new(0,0);
	gtk_widget_show(record_prefs);
	label=gtk_label_new("Recorder");
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(preferences_editpad),
				 record_prefs,
				 label);
	
	edit=varmanwidgets_entry_new("Recorder device",
				     "recorder",
				     global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(record_prefs),edit->visual,0,0,0);
	
	edit=varmanwidgets_entry_new("SCSI ID",
				     "rec_scsiid",
				     global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(record_prefs),edit->visual,0,0,0);

	edit=varmanwidgets_entry_new("recording client",
				     "rec_client",
				     global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(record_prefs),edit->visual,0,0,0);

	edit=varmanwidgets_entry_new("Fixating command",
				     "rec_fixate",
				     global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(record_prefs),edit->visual,0,0,0);

	edit=varmanwidgets_entry_new("Trackwrite command",
				     "rec_writetrack",
				     global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(record_prefs),edit->visual,0,0,0);
	
	edit=varmanwidgets_entry_new("Blank disc command",
				     "rec_cleardisc",
				     global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(record_prefs),edit->visual,0,0,0);	
	
	record_active=0; // cdwriter is initially not active	
	return rec;
}
;
	
	
	
	
	
