/* some helper functions for selection lists,
 * (used e.g. by filelist.c) */

#include <gtk/gtk.h>
#include <string.h>

#include "main.h"
#include "dndsetup.h"
#include "selectlist.h"
#include "doubleclick.h"

/* non-public callback functions */

static gint selectlist_popup(GtkWidget *w,
			     GdkEventButton *event,
			     selectlist_info *i)
{
	GtkWidget *menu;
	GtkWidget *entry;
	
	if (event->button==3)
	  {
		  menu=gtk_menu_new();

		  if (i->item_remove!=NULL)
		    {			    
			    entry=gtk_menu_item_new_with_label("Delete");
			    gtk_signal_connect_object(GTK_OBJECT(entry),"activate",
						      GTK_SIGNAL_FUNC(i->item_remove),(void*)&i->selection);
			    gtk_menu_append(GTK_MENU(menu),entry);
			    gtk_widget_show(entry);
		    }
		  ;
		  if (i->mkdir!=NULL)
		    {			    
			    entry=gtk_menu_item_new_with_label("Create directory");
			    gtk_signal_connect_object(GTK_OBJECT(entry),"activate",
						      GTK_SIGNAL_FUNC(i->mkdir),(void*)&i->header);
			    gtk_menu_append(GTK_MENU(menu),entry);
			    gtk_widget_show(entry);
		    }
		  ;
		  

		  gtk_menu_popup (GTK_MENU(menu),
				  NULL,NULL,
				  NULL,NULL,
				  3,
				  0);
	  }
	;
}
;


void selectlist_drag(GtkWidget *w,
		     GdkDragContext *ct,
		     GtkSelectionData *data,
		     guint inf,
		     guint time,
		     selectlist_info *info)
{
	gtk_selection_data_set (data,
			data->target,
			8,(char*)&info->selection,strlen((char*)&info->selection));
}
;

void selectlist_dragdone(GtkWidget *w,
			 GdkDragContext *c,
			 selectlist_info *info)
{
	gtk_clist_unselect_all(GTK_CLIST(w));
}
;
	

void selectlist_drop(GtkWidget *w,
	      GdkDragContext *c,
	      gint x,
	      gint y,
	      GtkSelectionData *data,
	      guint inf,
	      guint time,
	      selectlist_info *info)
{
	if ((data->length>=0) && (data->format == 8) 
	    && (info->receiver!=NULL))
	  {
		  info->receiver((gchar*)data->data);
		  gtk_drag_finish (c,TRUE,FALSE,time);
		  return;
	  }
	;
	gtk_drag_finish (c,FALSE,FALSE,time);
}
;

void selectlist_append(char *s,char *h,char *f) // selectionlist,header,file
{
	char *entry;
	char *cmpcopy;
	
	cmpcopy=(char*)malloc(32768);
	strcpy(cmpcopy,CRLF);
	strcat(cmpcopy,s);
	
	entry=(char*)malloc(1024);
	strcpy(entry,CRLF);
	strcat(entry,h);
	strcat(entry,f);       
	strcat(entry,CRLF); // entry=CRLF+header+file+CRLF
	
	if (strstr(cmpcopy,entry)==NULL)
	  {
		  strcat(s,h);
		  strcat(s,f);
		  strcat(s,CRLF);
	  }
	;
	free(entry);free(cmpcopy);
}
;

void selectlist_remove(char *s,char *h,char *f)  // see above
{
	char *entry;
	char *cmpcopy;
	char *foundpos;
	
	cmpcopy=(char*)malloc(32768);
	strcpy(cmpcopy,CRLF);
	strcat(cmpcopy,s);
	
	entry=(char*)malloc(1024);
	strcpy(entry,CRLF);
	strcat(entry,h);
	strcat(entry,f);       
	strcat(entry,CRLF); // entry=CRLF+header+file+CRLF
		
	foundpos=strstr(cmpcopy,entry);
	while (foundpos!=NULL)
	  {
		  strcpy(foundpos,(char*)((int)foundpos+strlen(entry)-2));
		  foundpos=strstr(cmpcopy,entry);
	  }
	;
	strcpy(s,&cmpcopy[2]);        // cut leading cr+lf
	free(entry);free(cmpcopy);
}
;
	

void selectlist_select(GtkCList *clist,gint row,gint column,
		       GdkEventButton *event, selectlist_info *info)
{
	char *filename;
	gtk_clist_get_text(clist,row,
			   info->significant_column,&filename); // get 1st column of clist
	selectlist_append((char*)&info->selection,
			  (char*)&info->header,filename);
}
;

void selectlist_unselect(GtkCList *clist,gint row,gint column,
		       GdkEventButton *event, selectlist_info *info)
{
	char *filename;
	gtk_clist_get_text(clist,row,info->significant_column,&filename);
	selectlist_remove((char*)&info->selection,
			  (char*)&info->header,filename);
}
;

selectlist_info *selectlist_info_create(char *header,int significant_column,
					selectlist_dropcallback receiver,
					int send,int popup,
					GtkSignalFunc item_remove,
					GtkSignalFunc mkdir,
					GtkSignalFunc doubleclick
					)
{
	selectlist_info *sel;
	
	sel=(selectlist_info*)malloc(sizeof(selectlist_info));
	sel->selection[0]=0;
	strcpy((char*)&sel->header,header);
	sel->significant_column=significant_column;
	sel->receiver=receiver;
	sel->send=send;
	sel->popup=popup;
	
	sel->item_remove=item_remove;
	sel->mkdir=mkdir;
	sel->doubleclick=doubleclick;
	return sel;
}
;

void selectlist_doubleclick(GtkWidget *w,GdkEventButton *event,
			   selectlist_info *info)
{
	if (info->doubleclick!=NULL)
	    info->doubleclick((void*)&info->selection);	
}
;

GtkWidget *selectlist_create(selectlist_info *sel,int columns)
{
	GtkWidget *selectlist;
		
	selectlist=gtk_clist_new(columns);
	sel->self=GTK_CLIST(selectlist);
	
	if (sel->send)           // let the user specify if the current window should be sourced
	    dndsetup_drag(selectlist,
			  GTK_SIGNAL_FUNC(selectlist_drag),
			  GTK_SIGNAL_FUNC(selectlist_dragdone),
			  sel);
	if (sel->receiver!=NULL) // do not install drop handler if no handler is specified
	    dndsetup_drop(selectlist,GTK_SIGNAL_FUNC(selectlist_drop),sel);
	if (sel->popup)
	    gtk_signal_connect(GTK_OBJECT(selectlist),"button_press_event",
			       GTK_SIGNAL_FUNC(selectlist_popup),sel);
	
	gtk_signal_connect(GTK_OBJECT(selectlist),"select_row",
			   GTK_SIGNAL_FUNC(selectlist_select),sel);
	gtk_signal_connect(GTK_OBJECT(selectlist),"unselect_row",
			   GTK_SIGNAL_FUNC(selectlist_unselect),sel);
	doubleclick_connect(GTK_WIDGET(selectlist),
			    GTK_SIGNAL_FUNC(selectlist_doubleclick),
			    sel);

	return selectlist;
}
;
		  
