/* This file implements the various input widgets for the preferences 
 * of Gnome Toaster. The most important thing about those widgets is that
 * all of them take a varman variable as argument,which is then directly
 * affected by any changes done to the input widget. */

/* IMPORTANT: input widgets must provide two different ways of processing 
 * informations. first:  apply all changes to the referring vars immediately.
 *               second: only apply changes when a special signal is received.*/

#include <gtk/gtk.h>

#include "varman.h"
#include "varmanwidgets.h"

void varmanwidgets_entry_apply(GtkWidget *w,
			       varmanwidgets_widget *d)
{
	varman_setvar(d->database,(char*)&d->varname,gtk_entry_get_text(GTK_ENTRY(d->edit)));
}
;

void varmanwidgets_entry_undo(GtkWidget *w,
			       varmanwidgets_widget *d)
{
	gtk_entry_set_text(GTK_ENTRY(d->edit),varman_getvar(d->database,d->varname));
}
;

void varmanwidgets_entry_txtch(GtkWidget *w,GdkEventKey *e,varmanwidgets_widget *d)
{
	varmanwidgets_entry_apply(w,d);
}
;

varmanwidgets_widget *varmanwidgets_entry_new(char *label,
					      char *var,
					      varman_database *database,
					      int apply_mode,
					      int labelsize)
{
	varmanwidgets_widget *widget;
		
	widget=(varmanwidgets_widget*)malloc(sizeof(varmanwidgets_widget));
	
	widget->visual=gtk_hbox_new(0,0);
	
	widget->label=gtk_label_new(label);
	if (labelsize>0)
	    gtk_widget_set_usize(widget->label,labelsize,-1);
	gtk_label_set_justify(GTK_LABEL(widget->label),GTK_JUSTIFY_LEFT);
	gtk_widget_show(widget->label);
	gtk_box_pack_start(GTK_BOX(widget->visual),widget->label,0,0,0);
	
	widget->edit=gtk_entry_new();	
	gtk_entry_set_text(GTK_ENTRY(widget->edit),varman_getvar(database,var));
	gtk_widget_show(widget->edit);
	gtk_box_pack_start(GTK_BOX(widget->visual),widget->edit,0,0,0);
	
	gtk_widget_show(widget->visual);
	
	widget->database=database;
	strcpy((char*)&widget->varname,var);
	
	switch (apply_mode)
	  {
	  case APPLYMODE_ALWAYS:
		  gtk_signal_connect(GTK_OBJECT(widget->edit),"key_press_event",
				     varmanwidgets_entry_txtch,widget);
		  break;
	  case APPLYMODE_BUTTON:
		  preferences_setapplyhandler(varmanwidgets_entry_apply,widget);
		  preferences_setundohandler(varmanwidgets_entry_undo,widget);
		  break;
	  }
	;	
	return widget;
}
;

void varmanwidgets_checkbox_apply(GtkWidget *w,
				  varmanwidgets_widget *d)
{
	varman_setvar(d->database,(char*)&d->varname,
		      (GTK_TOGGLE_BUTTON(d->edit)->active)?(char*)&d->activetext
		                                          :(char*)&d->inactivetext);
}
;

void varmanwidgets_checkbox_undo(GtkWidget *w,
			       varmanwidgets_widget *d)
{
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->edit),
				     (!strcmp(varman_getvar(d->database,(char*)&d->varname),
					      (char*)&d->activetext))?1:0);
}
;

varmanwidgets_widget *varmanwidgets_checkbox_new(char *label,
						 char *var,
						 varman_database *database,
						 int apply_mode,
						 int labelsize,
						 char *activetext,
						 char *inactivetext)
{
	varmanwidgets_widget *widget;
		
	widget=(varmanwidgets_widget*)malloc(sizeof(varmanwidgets_widget));
	
	widget->visual=gtk_hbox_new(0,0);
		
	widget->edit=gtk_check_button_new_with_label(label);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget->edit),
				     (!strcmp(varman_getvar(database,var),activetext))?1:0);
	gtk_widget_show(widget->edit);
	gtk_box_pack_start(GTK_BOX(widget->visual),widget->edit,0,0,0);
	
	gtk_widget_show(widget->visual);
	
	widget->database=database;
	strcpy((char*)&widget->varname,var);
	strcpy((char*)&widget->activetext,activetext);
	strcpy((char*)&widget->inactivetext,inactivetext);	
	
	switch (apply_mode)
	  {
	  case APPLYMODE_ALWAYS:
		  gtk_signal_connect(GTK_OBJECT(widget->edit),"clicked",
				     varmanwidgets_checkbox_apply,widget);
		  break;
	  case APPLYMODE_BUTTON:
		  preferences_setapplyhandler(varmanwidgets_checkbox_apply,widget);
		  preferences_setundohandler(varmanwidgets_checkbox_undo,widget);
		  break;
	  }
	;	
	return widget;
}
;


	
	
	
	
	
	
